import json
from functools import lru_cache
from pathlib import Path
from typing import Mapping

from datasets import load_dataset

from evoprompt.task import TextClassification
from evoprompt.task.task import DatasetDatum


class Subj(TextClassification):
    shorthand = "subj"

    def load_validation_set(
        self, validation_dataset: str | None, validation_split: str | None
    ):
        return load_dataset(
            "csv",
            data_files={"validation": "evoprompt/data/subj/train.csv"},
            split="validation",
            column_names=["label", "text"],
        )

    def load_test_set(self, test_dataset: str, test_split: str | None):
        # this dataset is not in correct format therefore we process it after reading as a simple text file
        return load_dataset(
            "csv",
            data_files={"test": "evoprompt/data/subj/test.csv"},
            split="test",
            column_names=["label", "text"],
        )

    def _get_text_for_datum(self, datum: DatasetDatum) -> str:
        return datum["text"]

    def _get_gold_label_for_datum(self, datum: DatasetDatum) -> str:
        return datum["label"]

    @lru_cache
    def _get_label_mapping(self) -> Mapping:
        classes = ["subjective", "objective"]
        return dict(zip(classes, range(len(classes))))

    @property
    def base_prompts(self):
        #  from the paper: RLPROMPT: Optimizing Discrete Text Prompts with Reinforcement Learning
        return [
            """In this task, you are given sentences from reviews. The task is to classify a sentence as "subjective" if the opinion of the sentence is subjective or as "objective" if the opinion of the sentence is objective."""
        ]

    @property
    def base_prompts(self):
        initial_prompts_file = Path("evoprompt/initial_prompts/subj/prompts.json")
        with initial_prompts_file.open() as json_file:
            return json.load(json_file)
